/*  
 *  This example writes data to the HDF5 file by rows.
 *  Number of processes is assumed to be 1 or multiples of 2 (up to 8)
 */
 
#include "hdf5.h"
#include "stdlib.h"

#define H5FILE_NAME     "SDS_row.h5"
#define DATASETNAME 	"IntArray" 
#define NX     8                     /* dataset dimensions */
#define NY     5
#define RANK   2

int
main (int argc, char **argv)
{
    /*
     * HDF5 APIs definitions
     */ 	
    hid_t       file_id, dset_id;       /* file and dataset identifiers */
    hid_t       dataspace_in_file, local_dataspace_in_memory;      /* file and memory dataspace identifiers */
    hsize_t     dimsfull[RANK];         /* full dataset dimensions */
    int         *local_data_in_memory;  /* pointer to local data buffer to write */
    
    /* hyperslab selection parameters */
    hsize_t     count[RANK];          /* local dataset dimensions */
    hsize_t     offset[RANK];         /* offset where to write the local hyperslab on disk */
    hid_t	    plist_id;             /* property list identifier */
    int         i;
    herr_t	status;

    /*
     * MPI variables
     */
    int mpi_size, mpi_rank;
    MPI_Comm comm  = MPI_COMM_WORLD;
    MPI_Info info  = MPI_INFO_NULL;

    /*
     * Initialize MPI
     */
    MPI_Init(&argc, &argv);
    MPI_Comm_size(comm, &mpi_size);
    MPI_Comm_rank(comm, &mpi_rank);  
 
    /* 
     * Set up file access property list with parallel I/O access
     */
    plist_id = H5Pcreate(H5P_FILE_ACCESS);
    H5Pset_fapl_mpio(plist_id, comm, info);

    /*
     * Create a new file collectively and release property list identifier.
     */
    file_id = H5Fcreate(H5FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT, plist_id);
    H5Pclose(plist_id);
   

    /*
     * Create the dataspace for the full dataset.
     */
    dimsfull[0] = NX;
    dimsfull[1] = NY;
    dataspace_in_file = H5Screate_simple(RANK, dimsfull, NULL); 

    /*
     * Create the full dataset on disk with default properties.
     */
    dset_id = H5Dcreate(file_id, DATASETNAME, H5T_NATIVE_INT, dataspace_in_file,
			H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

    /* 
     * Each process defines the local dataset in memory and the offset where to write 
     * the local hyperslab to the file.
     */
    count[0]  = dimsfull[0]/mpi_size;
    count[1]  = dimsfull[1];
    offset[0] = mpi_rank * count[0];
    offset[1] = 0;

    /* Create the local dataspace */
    local_dataspace_in_memory = H5Screate_simple(RANK, count, NULL);

    /*
     * Select hyperslab in the file. This changes the dataspace_in_file and tells it 
     * that we write a hyperslab at the given offset. By chosing something else than
     * H5S_SELECT_SET one can also combine different hyperslabs (with OR/AND/XOR etc.).
     * Remember that we set *stride and *block to NULL in our case.
     */
    H5Sselect_hyperslab(dataspace_in_file, H5S_SELECT_SET, offset, NULL, count, NULL);

    /*
     * Initialize local data buffer.
     */
    local_data_in_memory = (int *) malloc(sizeof(int)*count[0]*count[1]);
    for (i=0; i < count[0]*count[1]; i++) {
        local_data_in_memory[i] = mpi_rank + 10;
    }

    /*
     * Create property list for dataset write. With this XFER property list one can control e.g.
     * if you want collective writes/reads etc.
     */
    plist_id = H5Pcreate(H5P_DATASET_XFER);
   
   /*
    * Write hyperslab to file and free local memory
    */  
    status = H5Dwrite(dset_id, H5T_NATIVE_INT, local_dataspace_in_memory, dataspace_in_file,
		      plist_id, local_data_in_memory);
    free(local_data_in_memory);

    /*
     * Close/release resources.
     */
    H5Dclose(dset_id);
    H5Sclose(dataspace_in_file);
    H5Sclose(local_dataspace_in_memory);
    H5Pclose(plist_id);
    H5Fclose(file_id);
 
    MPI_Finalize();

    return 0;
}     
