/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * Copyright by the Board of Trustees of the University of Illinois.         *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of HDF5.  The full HDF5 copyright notice, including     *
 * terms governing use, modification, and redistribution, is contained in    *
 * the COPYING file, which can be found at the root of the source code       *
 * distribution tree, or in https://www.hdfgroup.org/licenses.               *
 * If you do not have access to either file, you may request a copy from     *
 * help@hdfgroup.org.                                                        *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*
 *  This example writes data to the HDF5 file.
 *  Data conversion is performed during write operation.
 */

#include "hdf5.h"

#define H5FILE_NAME "SDS.h5"
#define NX          5 /* dataset dimensions */
#define NY          6
#define RANK        2

int
main(void)
{   
    /* hid_t: handler for any HDF5 objects (file, groups, dataset, dataspace, datatypes...) */
    hid_t   file, dataset;  /* file, group and dataset handles */
    hid_t   dataspace;      /* handles */
    hid_t   mygroup;            
    /* hsize t: C type used for number of elements of a dataset (in each dimension) */
    hsize_t dimsf[RANK];    /* dataset dimensions */
    /* herr t: C type used for getting error status of HDF5 functions */
    herr_t  status;
    int     data[NX][NY]; /* data to write */
    int     i, j;

    /*
     * Data  and output buffer initialization.
     */
    for (j = 0; j < NX; j++)
        for (i = 0; i < NY; i++)
            data[j][i] = i + j;
    /*
     * 0 1 2 3 4 5
     * 1 2 3 4 5 6
     * 2 3 4 5 6 7
     * 3 4 5 6 7 8
     * 4 5 6 7 8 9
     */

    /*
     * Create a new file using H5F_ACC_TRUNC access,
     * default file creation properties, and default file
     * access properties. H5F_ACC_TRUNC means that an existing file will be overwritten,
     * if the file does not exist, it will be created and openend in read/write mode.
     */
    file = H5Fcreate(H5FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

    /*
     * Describe the size of the array and create the data space for fixed
     * size dataset. A simple dataspace, H5S_SIMPLE, is a multidimensional array of elements. 
     * For further explanation see e.g. http://davis.lbl.gov/Manuals/HDF5-1.8.7/UG/12_Dataspaces.html
     */
    dimsf[0]  = NX;
    dimsf[1]  = NY;
    dataspace = H5Screate_simple(RANK, dimsf, NULL);

    /*
    * Create a group in the file.
    */
    mygroup = H5Gcreate(file, "/myimportantdata", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);


    /*
     * Create a new dataset within the file using defined dataspace and
     * datatype and default dataset creation properties.
     */
    dataset = H5Dcreate(file, "/myimportantdata/IntArray", H5T_NATIVE_INT, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

    /*
     * Write the data to the dataset using default transfer properties.
     */
    status = H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);

    /*
     * Close/release resources.
     */
    H5Sclose(dataspace);
    H5Dclose(dataset);
    H5Gclose(mygroup);
    H5Fclose(file);

    return 0;
}
